@@ -1,8 +1,14 @@
-from __builtin__ import all as _all, any as _any
+import sys
+try:
+    from __builtin__ import all as _all, any as _any
+except ImportError:
+    from builtins import all as _all, any as _any
 from operator import itemgetter, methodcaller
 from collections import Mapping, Set, Iterable, Iterator, defaultdict
 from itertools import chain, tee
+from functools import reduce
 
+from .cross import basestring, xrange
 from .primitives import EMPTY
 from .funcs import identity, partial, compose, complement
 from .funcmakers import wrap_mapper, wrap_selector
@@ -33,8 +39,12 @@ def _factory(coll, mapper=None):
 def empty(coll):
     return _factory(coll)()
 
-def iteritems(coll):
-    return coll.iteritems() if hasattr(coll, 'iteritems') else coll
+if sys.version_info[0] == 2:
+    def iteritems(coll):
+        return coll.iteritems() if hasattr(coll, 'iteritems') else coll
+else:
+    def iteritems(coll):
+        return coll.items() if hasattr(coll, 'items') else coll
 
 
 def join(colls):
@@ -49,7 +59,10 @@ def join(colls):
     if isinstance(dest, basestring):
         return ''.join(colls)
     elif isinstance(dest, Mapping):
-        return reduce(lambda a, b: _factory(dest)(a, **b), colls)
+        result = dest.copy()
+        for d in it:
+            result.update(d)
+        return result
     elif isinstance(dest, Set):
         return dest.union(*it)
     elif isinstance(dest, (Iterator, xrange)):
@@ -77,11 +90,20 @@ def walk(f, coll):
 
 @wrap_mapper
 def walk_keys(f, coll):
-    return walk(lambda (k, v): (f(k), v), coll)
+    # NOTE: we use this awkward construct instead of lambda to be Python 3 compatible
+    def pair_f(pair):
+        k, v = pair
+        return f(k), v
+
+    return walk(pair_f, coll)
 
 @wrap_mapper
 def walk_values(f, coll):
-    pair_f = lambda (k, v): (k, f(v))
+    # NOTE: we use this awkward construct instead of lambda to be Python 3 compatible
+    def pair_f(pair):
+        k, v = pair
+        return k, f(v)
+
     return _factory(coll, mapper=f)(imap(pair_f, iteritems(coll)))
 
 # TODO: prewalk, postwalk and friends
@@ -92,11 +114,11 @@ def select(pred, coll):
 
 @wrap_selector
 def select_keys(pred, coll):
-    return select(lambda (k, v): pred(k), coll)
+    return select(lambda pair: pred(pair[0]), coll)
 
 @wrap_selector
 def select_values(pred, coll):
-    return select(lambda (k, v): pred(v), coll)
+    return select(lambda pair: pred(pair[1]), coll)
 
 
 def compact(coll):
@@ -149,7 +171,10 @@ def zipdict(keys, vals):
     return dict(zip(keys, vals))
 
 def flip(mapping):
-    return walk(lambda (k, v): (v, k), mapping)
+    def flip_pair(pair):
+        k, v = pair
+        return v, k
+    return walk(flip_pair, mapping)
 
 def project(mapping, keys):
     return _factory(mapping)((k, mapping[k]) for k in keys if k in mapping)
@@ -178,3 +203,15 @@ def pluck(key, mappings):
 
 def invoke(objects, name, *args, **kwargs):
     return map(methodcaller(name, *args, **kwargs), objects)
+
+
+### For Python 3
+
+def lwhere(mappings, **cond):
+    return list(where(mappings, **cond))
+
+def lpluck(key, mappings):
+    return list(pluck(key, mappings))
+
+def linvoke(objects, name, *args, **kwargs):
+    return list(invoke(objects, name, *args, **kwargs))
