@@ -1,8 +1,9 @@
+import sys
 from operator import add
-from itertools import islice, ifilter, imap, izip, chain, tee, ifilterfalse, dropwhile, takewhile, \
-                      groupby
+from itertools import islice, chain, tee, dropwhile, takewhile, groupby
 from collections import defaultdict, deque, Sequence
 
+from .cross import ifilter, imap, izip, ifilterfalse, xrange
 from .primitives import EMPTY
 from .types import is_seqcont
 from .funcs import partial
@@ -14,8 +15,8 @@ __all__ = [
     'take', 'drop', 'first', 'second', 'nth', 'last', 'rest', 'butlast', 'ilen',
     'map', 'filter', 'imap', 'ifilter', 'remove', 'iremove', 'keep', 'ikeep', 'without', 'iwithout',
     'concat', 'iconcat', 'chain', 'cat', 'icat', 'flatten', 'iflatten', 'mapcat', 'imapcat',
-    'izip', 'interleave', 'interpose', 'distinct',
-    'dropwhile', 'takewhile', 'split', 'split_at', 'split_by',
+    'izip', 'interleave', 'interpose', 'distinct', 'idistinct',
+    'dropwhile', 'takewhile', 'split', 'isplit', 'split_at', 'isplit_at', 'split_by', 'isplit_by',
     'group_by', 'group_by_keys', 'count_by',
     'partition', 'ipartition', 'chunks', 'ichunks', 'ipartition_by', 'partition_by',
     'with_prev', 'pairwise',
@@ -54,16 +55,13 @@ def nth(n, seq):
     return next(islice(seq, n, n+1), None)
 
 def last(seq):
-    if hasattr(seq, '__getslice__'):
-        try:
-            return seq[-1]
-        except IndexError:
-            return None
-
-    item = None
-    for item in iter(seq):
-        pass
-    return item
+    try:
+        return first(reversed(seq))
+    except TypeError:
+        item = None
+        for item in iter(seq):
+            pass
+        return item
 
 def rest(seq):
     return drop(1, seq)
@@ -91,11 +89,20 @@ def ilen(seq):
 
 # TODO: tree-seq equivalent
 
-map = wrap_mapper(map)
+
 imap = wrap_mapper(imap)
-filter = wrap_selector(filter)
 ifilter = wrap_selector(ifilter)
 
+if sys.version_info[0] == 2:
+    map = wrap_mapper(map)
+    filter = wrap_selector(filter)
+else:
+    def map(f, *seqs):
+        return list(imap(f, *seqs))
+    def filter(pred, seq):
+        return list(ifilter(pred, seq))
+
+
 def remove(pred, seq):
     return list(iremove(pred, seq))
 iremove = wrap_selector(ifilterfalse)
@@ -243,6 +250,8 @@ def _icut_iter(drop_tail, n, step, seq):
 def _icut(drop_tail, n, step, seq=EMPTY):
     if seq is EMPTY:
         step, seq = n, step
+    # NOTE: range() is capable of slicing in python 3,
+    #       so this implementation could be updated
     if isinstance(seq, Sequence) and not isinstance(seq, xrange):
         return _icut_seq(drop_tail, n, step, seq)
     else:
